@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import contextlib
+import copy
 import importlib.util
 import json
 import logging
@@ -11,10 +12,7 @@ from typing import TYPE_CHECKING, Any, ClassVar, Literal, cast, get_origin
 from pydantic import BaseModel, PrivateAttr, model_serializer
 from pydantic import ConfigDict as _ConfigDict
 from pydantic.main import IncEx
-from typing_extensions import TypedDict, Unpack, override
-
-from .missing import MISSING as _MISSING
-from .missing import validate_no_missing_values
+from typing_extensions import Self, TypedDict, Unpack, override
 
 if TYPE_CHECKING:
     from ruamel.yaml import YAML
@@ -27,9 +25,6 @@ if TYPE_CHECKING:
     _MutableMappingBase = object
 
 
-_DraftConfigContextSentinel = object()
-
-
 class ConfigDict(_ConfigDict, total=False):
     repr_diff_only: bool
     """
@@ -43,6 +38,15 @@ class ConfigDict(_ConfigDict, total=False):
     Defaults to `True`.
     """
 
+    set_default_hash: bool
+    """
+    Whether to set the default hash function for the config class. By default,
+    Pydantic adds a `__hash__` method for frozen models, but Config classes are mutable,
+    so we don't get a hash function by default. Setting this to `True` will enable the hash function
+    for the config class, which is useful for using the config class as a key in a dictionary.
+    Defaults to `True`.
+    """
+
 
 class DumpKwargs(TypedDict, total=False):
     include: IncEx | None
@@ -117,15 +121,7 @@ class Config(BaseModel, _MutableMappingBase):
         ```
     """
 
-    MISSING: ClassVar[Any] = _MISSING
-    """
-    Alias for the `MISSING` constant.
-    """
-
-    model_config: ClassVar[ConfigDict] = ConfigDict(  # type: ignore
-        # By default, Pydantic will throw a warning if a field starts with "model_",
-        # so we need to disable that warning (beacuse "model_" is a popular prefix for ML).
-        protected_namespaces=(),
+    model_config: ClassVar[ConfigDict] = ConfigDict(  # pyright: ignore[reportIncompatibleVariableOverride]
         validate_assignment=True,
         validate_return=True,
         validate_default=True,
@@ -135,13 +131,53 @@ class Config(BaseModel, _MutableMappingBase):
         extra="ignore",
         validation_error_cause=True,
         use_attribute_docstrings=True,
+        # Our custom config options
+        repr_diff_only=False,
+        no_validate_assignment_for_draft=True,
+        set_default_hash=True,
     )
 
     if TYPE_CHECKING:
 
-        def __init_subclass__(cls, **kwargs: Unpack[ConfigDict]):
+        @override
+        def __init_subclass__(cls, **kwargs: Unpack[ConfigDict]) -> None:
             super().__init_subclass__(**kwargs)
 
+    @override
+    @classmethod
+    def __pydantic_init_subclass__(cls, **kwargs) -> None:
+        super().__pydantic_init_subclass__(**kwargs)
+
+        if cls.model_config.get("set_default_hash", True):
+            _try_set_hash(cls)
+
+        from .registry import Registry, extract_registries_from_field_info
+
+        log.debug(f"__pydantic_init_subclass__: {cls}")
+
+        # Look through fields for registry annotations including nested types
+        registries: list[Registry] = []
+        registry_ids: set[int] = set()
+        for name, field in cls.__pydantic_fields__.items():
+            if not (found_registries := extract_registries_from_field_info(field)):
+                continue
+
+            for registry in found_registries:
+                if (id_ := id(registry)) in registry_ids:
+                    continue
+
+                log.debug(f"__pydantic_init_subclass__@{cls} - {name}: {registry}")
+                registries.append(registry)
+                registry_ids.add(id_)
+
+        # Register rebuild callback with each registry found
+        def _cb(_: Any):
+            cls.model_rebuild(force=True, raise_errors=False)
+            log.info(f"Rebuilt {cls} schema due to registry changes.")
+
+        for registry in registries:
+            registry._on_register_callbacks.append(_cb)
+
     def __draft_pre_init__(self):
         """Called right before a draft config is finalized."""
         pass
@@ -150,6 +186,10 @@ class Config(BaseModel, _MutableMappingBase):
         """Called after the final config is validated."""
         pass
 
+    def __after_post_init__(self):
+        """Called after __post_init__ is successfully called."""
+        pass
+
     def model_deep_validate(self, strict: bool = True):
         """
         Validate the config and all of its sub-configs.
@@ -161,10 +201,6 @@ class Config(BaseModel, _MutableMappingBase):
         config_dict = self.model_dump(round_trip=True)
         config = self.model_validate(config_dict, strict=strict)
 
-        # Make sure that this is not a draft config
-        if config._is_draft_config:
-            raise ValueError("Draft configs are not valid. Call `finalize` first.")
-
         return config
 
     @classmethod
@@ -183,98 +219,50 @@ class Config(BaseModel, _MutableMappingBase):
         # Then, we dump the config to a dict and then re-validate it
         return self.model_deep_validate(strict=strict)
 
+    _patched_post_init: ClassVar[bool] = False
+
+    @classmethod
+    @contextlib.contextmanager
+    def _nshconfig_patch_model_post_init(cls):
+        cls._patched_post_init = True
+        try:
+            yield
+        finally:
+            cls._patched_post_init = False
+
     @override
     def model_post_init(self, __context: Any) -> None:
         super().model_post_init(__context)
 
         # Call the `__post_init__` method if this is not a draft config
-        if __context is _DraftConfigContextSentinel:
+        if self._patched_post_init:
             return
 
         self.__post_init__()
 
-        # After `_post_init__` is called, we perform the final round of validation
-        self.model_post_init_validate()
+        self.__after_post_init__()
 
-    def model_post_init_validate(self):
-        validate_no_missing_values(self)
+    def model_validate_no_missing(self):
+        from .missing import validate_no_missing
+
+        validate_no_missing(self)
 
     @classmethod
     def model_construct_draft(cls, _fields_set: set[str] | None = None, **values: Any):
         """
-        NOTE: This is a copy of the `model_construct` method from Pydantic's `Model` class,
+        This is a copy of the `model_construct` method from Pydantic's `Model` class,
             with the following changes:
             - The `model_post_init` method is called with the `_DraftConfigContext` context.
             - The `_is_draft_config` attribute is set to `True` in the `values` dict.
-
-        Creates a new instance of the `Model` class with validated data.
-
-        Creates a new model setting `__dict__` and `__pydantic_fields_set__` from trusted or pre-validated data.
-        Default values are respected, but no other validation is performed.
-
-        !!! note
-            `model_construct()` generally respects the `model_config.extra` setting on the provided model.
-            That is, if `model_config.extra == 'allow'`, then all extra passed values are added to the model instance's `__dict__`
-            and `__pydantic_extra__` fields. If `model_config.extra == 'ignore'` (the default), then all extra passed values are ignored.
-            Because no validation is performed with a call to `model_construct()`, having `model_config.extra == 'forbid'` does not result in
-            an error if extra values are passed, but they will be ignored.
-
-        Args:
-            _fields_set: The set of field names accepted for the Model instance.
-            values: Trusted or pre-validated data dictionary.
-
-        Returns:
-            A new instance of the `Model` class with validated data.
         """
 
+        values = copy.deepcopy(values)
         values["_is_draft_config"] = True
-
-        m = cls.__new__(cls)
-        fields_values: dict[str, Any] = {}
-        fields_set = set()
-
-        for name, field in cls.model_fields.items():
-            if field.alias and field.alias in values:
-                fields_values[name] = values.pop(field.alias)
-                fields_set.add(name)
-            elif name in values:
-                fields_values[name] = values.pop(name)
-                fields_set.add(name)
-            elif not field.is_required():
-                fields_values[name] = field.get_default(call_default_factory=True)
-        if _fields_set is None:
-            _fields_set = fields_set
-
-        _extra: dict[str, Any] | None = None
-        if cls.model_config.get("extra") == "allow":
-            _extra = {}
-            for k, v in values.items():
-                _extra[k] = v
-        object.__setattr__(m, "__dict__", fields_values)
-        object.__setattr__(m, "__pydantic_fields_set__", _fields_set)
-        if not cls.__pydantic_root_model__:
-            object.__setattr__(m, "__pydantic_extra__", _extra)
-
-        if cls.__pydantic_post_init__:
-            m.model_post_init(_DraftConfigContextSentinel)
-            # update private attributes with values set
-            if (
-                hasattr(m, "__pydantic_private__")
-                and m.__pydantic_private__ is not None
-            ):
-                for k, v in values.items():
-                    if k in m.__private_attributes__:
-                        m.__pydantic_private__[k] = v
-
-        elif not cls.__pydantic_root_model__:
-            # Note: if there are any private attributes, cls.__pydantic_post_init__ would exist
-            # Since it doesn't, that means that `__pydantic_private__` should be set to None
-            object.__setattr__(m, "__pydantic_private__", None)
-
-        return m
+        with cls._nshconfig_patch_model_post_init():
+            return cls.model_construct(_fields_set, **values)
 
     @contextlib.contextmanager
-    def __patch_validator_validate_assignment(self):
+    def _nshconfig_patch_validator_validate_assignment(self):
         prev_value = self.model_config.get("validate_assignment", _notset := object())
         try:
             # We temporarily disable the validation of assignments
@@ -297,7 +285,9 @@ class Config(BaseModel, _MutableMappingBase):
                 if self._is_draft_config and self.model_config.get(
                     "no_validate_assignment_for_draft", True
                 ):
-                    stack.enter_context(self.__patch_validator_validate_assignment())
+                    stack.enter_context(
+                        self._nshconfig_patch_validator_validate_assignment()
+                    )
                 return super().__setattr__(name, value)
 
     @override
@@ -422,9 +412,12 @@ class Config(BaseModel, _MutableMappingBase):
     @model_serializer(mode="wrap")
     def include_literals(self, next_serializer):
         """Include fields with the `Literal` annotation in the dumped data,
-        even if `exclude_defaults` is `True`."""
+        even if `exclude_defaults` is `True`.
+
+        See https://github.com/pydantic/pydantic/discussions/9108#discussioncomment-8926452
+        for the original implementation."""
         dumped = next_serializer(self)
-        for name, field_info in self.model_fields.items():
+        for name, field_info in type(self).__pydantic_fields__.items():
             if get_origin(field_info.annotation) == Literal:
                 dumped[name] = getattr(self, name)
         return dumped
@@ -443,6 +436,25 @@ class Config(BaseModel, _MutableMappingBase):
         """
         return cls.model_validate(config_dict)
 
+    @classmethod
+    def from_dict_or_instance(cls, data: Mapping[str, Any] | Self):
+        """Create configuration from a dictionary or an instance.
+
+        Args:
+            data: Dictionary containing configuration values or an instance of the config
+
+        Returns:
+            A validated configuration instance
+        """
+        if isinstance(data, cls):
+            return data
+        elif isinstance(data, Mapping):
+            return cls.from_dict(dict(data))
+        else:
+            raise TypeError(
+                "data must be a dictionary or an instance of the config class"
+            )
+
     def to_dict(self) -> dict[str, Any]:
         """Convert configuration to a dictionary.
 
@@ -656,6 +668,49 @@ class Config(BaseModel, _MutableMappingBase):
 
         return parse_yaml_file_as(cls, path)
 
+    @classmethod
+    def from_python_file(cls, path: str | Path, /):
+        """Create configuration from a Python file.
+
+        The Python file should export a `__config__` variable that contains the configuration,
+        or a `__create_config__` function that returns a configuration.
+
+        Args:
+            path: Path to the Python file
+
+        Returns:
+            A validated configuration instance
+
+        Raises:
+            FileNotFoundError: If the Python file does not exist
+            ImportError: If the Python file cannot be imported
+            ValueError: If the Python file does not export a valid `__config__` variable
+        """
+        from .utils import import_and_parse_python_file
+
+        return import_and_parse_python_file(path, cls)
+
+    @classmethod
+    def from_python_module(cls, module_name: str, /):
+        """Create configuration from a Python module.
+
+        The Python module should export a `__config__` variable that contains the configuration,
+        or a `__create_config__` function that returns a configuration.
+
+        Args:
+            module_name: Name of the Python module (e.g. "myapp.config")
+
+        Returns:
+            A validated configuration instance
+
+        Raises:
+            ImportError: If the Python module cannot be imported
+            ValueError: If the Python module does not export a valid `__config__` variable
+        """
+        from .utils import import_and_parse_python_module
+
+        return import_and_parse_python_module(module_name, cls)
+
     # endregion
 
 
@@ -667,20 +722,45 @@ def _get_schema_uri(cls: type[Config]) -> str | None:
 
     from .export import find_config_metadata
 
-    if metadata := find_config_metadata(cls):
-        _, schema_path = metadata
-        if schema_path is not None:
-            # Get the absolute path by looking up from the metadata file location
-            if (
-                spec := importlib.util.find_spec(cls.__module__)
-            ) is None or spec.origin is None:
-                return None
-            module_dir = Path(spec.origin).parent
-            # Look up until we find the .nshconfig.generated.json file
-            current_dir = module_dir
-            while current_dir.parent != current_dir:
-                if (current_dir / ".nshconfig.generated.json").exists():
-                    file_path = str((current_dir / schema_path).resolve().absolute())
-                    return f"file://{file_path}"
-                current_dir = current_dir.parent
+    if not (metadata := find_config_metadata(cls)):
+        return None
+
+    _, schema_path = metadata
+    if schema_path is None:
+        return None
+
+    # Get the absolute path by looking up from the metadata file location
+    if (
+        spec := importlib.util.find_spec(cls.__module__)
+    ) is None or spec.origin is None:
+        return None
+
+    module_dir = Path(spec.origin).parent
+    # Look up until we find the .nshconfig.generated.json file
+    current_dir = module_dir
+    while current_dir.parent != current_dir:
+        if (current_dir / ".nshconfig.generated.json").exists():
+            file_path = str((current_dir / schema_path).resolve().absolute())
+            return f"file://{file_path}"
+        current_dir = current_dir.parent
+
     return None
+
+
+def _try_set_hash(cls: type[Config]):
+    # This is a bit of a hack as we're relying on Pydantic's internal API,
+    # but we can fail gracefully if it doesn't work.
+
+    try:
+        from pydantic._internal._model_construction import set_default_hash_func
+    except ImportError:
+        log.warning(
+            "Could not set default hash function for config class. "
+            "This is likely due to a Pydantic version that does not support this feature. "
+            "Please make sure you're using Pydantic >= 2.10. "
+            "If you are and still see this error, please report it to the nshconfig maintainers."
+        )
+        return
+
+    set_default_hash_func(cls, cls.__bases__)
+    log.info(f"Default hash function set for class: {cls.__name__}")
